package br.usp.nds.remote.integracao;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.persistence.EntityManager;

import br.usp.nds.remote.persistencia.GeracaoEvento;
import br.usp.nds.remote.persistencia.Leitura;
import br.usp.nds.remote.persistencia.Persistencia;

public class GeradorEventos {

	private static GeradorEventos instance;
	private List<GeracaoEvento> geradores = new LinkedList();
	private Map<GeracaoEvento, Timer> timers = new HashMap();
	
	public static void main(String[] args) {
		System.out.println(System.currentTimeMillis() / 1000);
	}
	
	public static GeradorEventos getInstance() {
		if (instance == null) {
			instance = new GeradorEventos();
		}
		return instance;
	}

	public void carregar() {
		carregarEventos();
		processarGeradores();
	}
	
	public void processarLeitura(Leitura leitura) {
		for (GeracaoEvento gerador : timers.keySet()) {
			if (gerador.getSensor().getCodigo() == leitura.getSensor().getCodigo()) {
				if (leitura.getValorConvertido().compareTo(gerador.getValorMaximo()) < 0 && leitura.getValorConvertido().compareTo(gerador.getValorMinimo()) > 0) {
					timers.get(gerador).cancel();
					timers.remove(gerador);
				}
			}
		}
		for (GeracaoEvento gerador : geradores) {
			if (gerador.getSensor().getCodigo() == leitura.getSensor().getCodigo()) {
				if (leitura.getValorConvertido().compareTo(gerador.getValorMaximo()) < 0 && leitura.getValorConvertido().compareTo(gerador.getValorMinimo()) > 0) {
					if (gerador.getEventoGerado() != null && gerador.getEventoGerado()) {
						gerador.eventoNaoGerado();
					}
				}
			}
		}
		for (final GeracaoEvento gerador : geradores) {
			if (!timers.containsKey(gerador) && gerador.getSensor().getCodigo() == leitura.getSensor().getCodigo()) {
				if (leitura.getValorConvertido().compareTo(gerador.getValorMaximo()) > 0 || leitura.getValorConvertido().compareTo(gerador.getValorMinimo()) < 0) {
					if ((gerador.getEventoGerado() == null || !gerador.getEventoGerado())) {
						System.out.println("schedulando evento...");
						Timer timer = new Timer();
						gerador.setUltimaLeitura(leitura.getCodigo());
						timer.schedule(new TimerTask() {

							@Override
							public void run() {
								System.out.println("gerando evento...");
								gerador.gerarEvento();
							}
						}, 1000 * gerador.getTempoSegundos());
						timers.put(gerador, timer);						
					} else {
						gerador.setUltimaLeitura(leitura.getCodigo());
					}
				}
			}
		}
	}
	
	private void processarGeradores() {
		for (Timer timer : timers.values()) {
			timer.cancel();
		}
		timers.clear();
		EntityManager em = Persistencia.getInstance().getEMF().createEntityManager();
		System.out.println("processando cada gerador...");
		for (final GeracaoEvento gerador : geradores) {
			if (gerador.getEventoGerado() != null && gerador.getEventoGerado()) continue;
			List<Object[]> leituras = em.createNativeQuery("select date_part('epoch', a.instante), l.valorconvertido, l.codigo "
					+ "from leitura l "
					+ "join aquisicao a on l.aquisicao_codigo = a.codigo "
					+ "where l.sensor_codigo = " + gerador.getSensor().getCodigo() + " "
					+ "and date_part('epoch', instante) > " + (System.currentTimeMillis() / 1000 - gerador.getTempoSegundos()) + " "
					+ "order by a.instante").getResultList();
			long instante = -1;
			int ultimaLeitura = 0;
			for (Object[] leitura : leituras) {
				if (new BigDecimal(leitura[1].toString()).compareTo(gerador.getValorMaximo()) >= 0 || new BigDecimal(leitura[1].toString()).compareTo(gerador.getValorMinimo()) <= 0) {
					if (instante == -1) {
						instante = Long.parseLong(leitura[0].toString());
					}
					ultimaLeitura = Integer.parseInt(leitura[2].toString());
				} else if (instante != -1) {
					ultimaLeitura = 0;
					instante = -1;
				}
			}
			if (instante != -1) {
				Timer timer = new Timer();
				gerador.setUltimaLeitura(ultimaLeitura);
				timer.schedule(new TimerTask() {

					@Override
					public void run() {
						gerador.gerarEvento();
					}
				}, 1000 * gerador.getTempoSegundos() - (System.currentTimeMillis() - instante * 1000));
				timers.put(gerador, timer);
			}
		}
		System.out.println("acabou processando cada gerador...");
		em.close();
	}
	
	private void carregarEventos() {
		EntityManager em = Persistencia.getInstance().getEMF().createEntityManager();
		GeracaoEvento e = new GeracaoEvento();
		e.injetar(em);
		geradores = e.listar();
		em.close();
	}
	
	private GeradorEventos() {
	}
}
