package br.usp.nds.remote.integracao;

import java.io.StringReader;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.json.Json;
import javax.json.JsonArray;
import javax.json.JsonObject;
import javax.json.JsonReader;
import javax.persistence.EntityManager;

import br.usp.nds.remote.persistencia.Aquisicao;
import br.usp.nds.remote.persistencia.Leitura;
import br.usp.nds.remote.persistencia.Mensagem;
import br.usp.nds.remote.persistencia.Persistencia;
import br.usp.nds.remote.persistencia.Sensor;
import br.usp.nds.remote.servico.WebSocketManager;

public class ProcessadorMensagem {

	public static void main(String[] args) {
		new ProcessadorMensagem()
				.registrar("{"
						+ "data: '20160302', "
						+ "hora: '141000', "
						+ "latitude: '23.45', "
						+ "longitude: '-30', "
						+ "sensores: [{"
							+ "codigo: '9', "
							+ "valor: '50'"
						+ "}, {"
							+ "codigo: '10', "
							+ "valor: '50'"
						+ "}, {"
							+ "codigo: '11', "
							+ "valor: '50'"
						+ "}]"
					+ "}")
				.processar()
				.verificarEventos();
	}
	
	private Mensagem mensagem;
	private List<Leitura> leituras = new LinkedList();
	
	public ProcessadorMensagem registrar(String payload) {
		//System.out.println(payload);
		EntityManager em = Persistencia.getInstance().getEMF().createEntityManager();
		mensagem = new Mensagem();
		mensagem.setData(new Date());
		mensagem.setMensagem(payload);
		em.getTransaction().begin();
		em.persist(mensagem);
		em.getTransaction().commit();
		em.close();
		WebSocketManager.getInstance().registrarMensagem(payload);
		return this;
	}
	
	public ProcessadorMensagem processar() {
		JsonReader reader = Json.createReader(new StringReader(mensagem.getMensagem()));
		JsonObject root = reader.readObject();
		EntityManager em = Persistencia.getInstance().getEMF().createEntityManager();
		em.getTransaction().begin();
		Date instante = null;
		try {
			instante = new SimpleDateFormat("yyyyMMddHHmmss").parse(root.getString("data") + "" + root.getString("hora"));
		} catch (ParseException e) {
			e.printStackTrace();
		}
		Aquisicao aquisicao = new Aquisicao(instante,
				mensagem.getData(), new Date(), new BigDecimal(root.getString("latitude")), new BigDecimal(root.getString("longitude")));
		em.persist(aquisicao);
		JsonArray sensores = root.getJsonArray("sensores");
		for (int i = 0; i < sensores.size(); i++) {
			Leitura leitura = new Leitura(aquisicao, em.find(Sensor.class, Integer.parseInt(((JsonObject)sensores.get(i)).getString("codigo"))), new BigDecimal(((JsonObject)sensores.get(i)).getString("valor"))).converterValor();
			em.persist(leitura);
			leituras.add(leitura);
		}
		em.getTransaction().commit();
		em.close();
		reader.close();
		return this;
	}
	
	public void verificarEventos() {
		for (Leitura leitura : leituras) {
			GeradorEventos.getInstance().processarLeitura(leitura);
		}
	}
}
