package br.usp.nds.remote.persistencia;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

@Entity
public class Sensor extends Entidade implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int codigo;
	private String nome;
	private String descricao;
	private BigDecimal coeficienteAngular;
	private BigDecimal coeficienteLinear;
	private BigDecimal valorMaximo;
	private BigDecimal valorMinimo;
	@ManyToOne
	private Embarcacao embarcacao = new Embarcacao();
	@ManyToOne
	private Subsistema subsistema = new Subsistema();
	@ManyToOne
	private TipoSensor tipo = new TipoSensor();
	@Transient
	private String codigosEmbarcacoes;
	
	public List<Sensor> listar() {
		String criterio = "";
		if (!(codigosEmbarcacoes != null && !codigosEmbarcacoes.isEmpty())) {
			codigosEmbarcacoes = "0";
		}
		criterio += " AND s.embarcacao.codigo IN (" + codigosEmbarcacoes + ") ";
		if (tipo.getCodigo() != 0) {
			criterio += " AND s.tipo.codigo = " + tipo.getCodigo() + " "; 
		}
		return em.createQuery("SELECT s FROM Sensor s WHERE 1 = 1 " + criterio + " ORDER BY s.embarcacao.nome, s.tipo").getResultList();		
	}

	public void incluir() {
		embarcacao = em.find(Embarcacao.class, embarcacao.getCodigo());
		subsistema = em.find(Subsistema.class, subsistema.getCodigo());
		tipo = em.find(TipoSensor.class, tipo.getCodigo());
		em.persist(this);
	}
	
	public void alterar() {
		embarcacao = em.find(Embarcacao.class, embarcacao.getCodigo());
		subsistema = em.find(Subsistema.class, subsistema.getCodigo());
		tipo = em.find(TipoSensor.class, tipo.getCodigo());
		em.merge(this);
	}
	
	public void excluir() {
		em.remove(em.find(Sensor.class, codigo));
	}
	
	public BigDecimal getValorMaximo() {
		return valorMaximo;
	}

	public void setValorMaximo(BigDecimal valorMaximo) {
		this.valorMaximo = valorMaximo;
	}

	public BigDecimal getValorMinimo() {
		return valorMinimo;
	}

	public void setValorMinimo(BigDecimal valorMinimo) {
		this.valorMinimo = valorMinimo;
	}

	public TipoSensor getTipo() {
		return tipo;
	}
	public void setTipo(TipoSensor tipo) {
		this.tipo = tipo;
	}
	public Subsistema getSubsistema() {
		return subsistema;
	}
	public void setSubsistema(Subsistema subsistema) {
		this.subsistema = subsistema;
	}
	public Embarcacao getEmbarcacao() {
		return embarcacao;
	}
	public void setEmbarcacao(Embarcacao embarcacao) {
		this.embarcacao = embarcacao;
	}
	public int getCodigo() {
		return codigo;
	}
	public void setCodigo(int codigo) {
		this.codigo = codigo;
	}
	public String getNome() {
		return nome;
	}
	public void setNome(String nome) {
		this.nome = nome;
	}
	public String getDescricao() {
		return descricao;
	}
	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}
	public BigDecimal getCoeficienteAngular() {
		return coeficienteAngular;
	}
	public void setCoeficienteAngular(BigDecimal coeficienteAngular) {
		this.coeficienteAngular = coeficienteAngular;
	}
	public BigDecimal getCoeficienteLinear() {
		return coeficienteLinear;
	}
	public void setCoeficienteLinear(BigDecimal coeficienteLinear) {
		this.coeficienteLinear = coeficienteLinear;
	}
	public String getCodigosEmbarcacoes() {
		return codigosEmbarcacoes;
	}
	public void setCodigosEmbarcacoes(String codigosEmbarcacoes) {
		this.codigosEmbarcacoes = codigosEmbarcacoes;
	}
}
